@@ -30,6 +30,11 @@ If you have questions concerning this license or the applicable additional terms
 
 #include "precompiled.h"
 #pragma hdrstop
+#if defined(_MSC_VER) && !defined(USE_OPENAL)
+	#include <sound/XAudio2/XA2_CinematicAudio.h>
+#else
+	#include <sound/OpenAL/AL_CinematicAudio.h>
+#endif
 
 
 extern idCVar s_noSound;
@@ -63,7 +68,10 @@ extern "C"
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 #include <libswscale/swscale.h>
+#include <libswresample/swresample.h>
+#include <libavutil/imgutils.h>
 }
+#define NUM_LAG_FRAMES 15	// SRS - Lag audio by 15 frames (~1/2 sec at 30 fps) for ffmpeg bik decoder AV sync
 #endif
 
 #ifdef USE_BINKDEC
@@ -90,24 +98,42 @@ public:
 	bool                    IsPlaying() const;
 	// RB end
 	virtual void			Close();
+	// SRS begin
+	virtual int             GetStartTime();
+	// SRS end
 	virtual void			ResetTime( int time );
 
 private:
 
 #if defined(USE_FFMPEG)
 	int						video_stream_index;
+	int						audio_stream_index; //GK: Make extra indexer for audio
 	AVFormatContext*		fmt_ctx;
 	AVFrame*				frame;
 	AVFrame*				frame2;
+	AVFrame*				frame3; //GK: make extra frame for audio
+#if LIBAVCODEC_VERSION_MAJOR > 58
+	const AVCodec*			dec;
+	const AVCodec*			dec2;	// SRS - Separate decoder for audio
+#else
 	AVCodec*				dec;
+	AVCodec*				dec2;	// SRS - Separate decoder for audio
+#endif
 	AVCodecContext*			dec_ctx;
+	AVCodecContext*			dec_ctx2;
 	SwsContext*				img_convert_ctx;
 	bool					hasFrame;
 	long					framePos;
-
+	AVSampleFormat			dst_smp;
+	bool					hasplanar;
+	SwrContext*				swr_ctx;
 	cinData_t				ImageForTimeFFMPEG( int milliseconds );
 	bool					InitFromFFMPEGFile( const char* qpath, bool looping );
 	void					FFMPEGReset();
+	uint8_t*				lagBuffer[NUM_LAG_FRAMES] = {};
+	int						lagBufSize[NUM_LAG_FRAMES] = {};
+	int						lagIndex;
+	bool					skipLag;
 #endif
 #ifdef USE_BINKDEC
 	BinkHandle				binkHandle;
@@ -116,11 +142,15 @@ private:
 	void					BinkDecReset();
 
 	YUVbuffer				yuvBuffer;
+	bool                    hasFrame;
 	int						framePos;
 	int						numFrames;
 	idImage*				imgY;
 	idImage*				imgCr;
 	idImage*				imgCb;
+	uint32_t				audioTracks;
+	uint32_t				trackIndex;
+	AudioInfo				binkInfo;
 #endif
 	idImage*				img;
 	bool					isRoQ;
@@ -189,6 +219,9 @@ private:
 	void					RoQPrepMcomp( int xoff, int yoff );
 	void					RoQReset();
 	// RB end
+
+	//GK:Also init variables for XAudio2 or OpenAL (SRS - this must be an instance variable)
+	CinematicAudio*			cinematicAudio;
 };
 
 // Carl: ROQ files from original Doom 3
@@ -233,13 +266,6 @@ idCinematicLocal::InitCinematic
 // RB: 64 bit fixes, changed long to int
 void idCinematic::InitCinematic()
 {
-#if defined(USE_FFMPEG)
-	// Carl: ffmpeg for Bink and regular video files
-	//common->Warning( "Loading FFMPEG...\n" );
-	avcodec_register_all();
-	av_register_all();
-#endif
-
 	// Carl: Doom 3 ROQ:
 	float t_ub, t_vr, t_ug, t_vg;
 	int i;
@@ -332,7 +358,7 @@ idCinematic::GetStartTime
 */
 int idCinematic::GetStartTime()
 {
-	return -1;
+	return -1;  // SRS - this is just the abstract virtual method
 }
 
 /*
@@ -406,29 +432,41 @@ idCinematicLocal::idCinematicLocal()
 	qStatus[0] = ( byte** )Mem_Alloc( 32768 * sizeof( byte* ), TAG_CINEMATIC );
 	qStatus[1] = ( byte** )Mem_Alloc( 32768 * sizeof( byte* ), TAG_CINEMATIC );
 
+	isRoQ = false;      // SRS - Initialize isRoQ for all cases, not just FFMPEG
 #if defined(USE_FFMPEG)
 	// Carl: ffmpeg stuff, for bink and normal video files:
-	isRoQ = false;
-//	fmt_ctx = avformat_alloc_context();
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
 	frame = av_frame_alloc();
 	frame2 = av_frame_alloc();
+	frame3 = av_frame_alloc();
 #else
 	frame = avcodec_alloc_frame();
 	frame2 = avcodec_alloc_frame();
+	frame3 = avcodec_alloc_frame();
 #endif // LIBAVCODEC_VERSION_INT
 	dec_ctx = NULL;
+	dec_ctx2 = NULL;
 	fmt_ctx = NULL;
 	video_stream_index = -1;
+	audio_stream_index = -1;
+	hasplanar = false;
+	swr_ctx = NULL;
 	img_convert_ctx = NULL;
 	hasFrame = false;
+	framePos = -1;
+	lagIndex = 0;
+	skipLag = false;
 #endif
 
 #ifdef USE_BINKDEC
 	binkHandle.isValid = false;
 	binkHandle.instanceIndex = -1; // whatever this is, it now has a deterministic value
+	hasFrame = false;
 	framePos = -1;
 	numFrames = 0;
+	audioTracks = 0;
+	trackIndex = -1;
+	binkInfo = {};
 
 	imgY = globalImages->AllocStandaloneImage( "_cinematicY" );
 	imgCr = globalImages->AllocStandaloneImage( "_cinematicCr" );
@@ -473,6 +511,12 @@ idCinematicLocal::idCinematicLocal()
 		img->AllocImage( opts, TF_LINEAR, TR_REPEAT );
 	}
 
+	//GK: Make sure the cinematic voices are the first to be initialized
+#if defined(_MSC_VER) && !defined(USE_OPENAL)
+	cinematicAudio = new( TAG_AUDIO ) CinematicAudio_XAudio2;
+#else
+	cinematicAudio = new( TAG_AUDIO ) CinematicAudio_OpenAL;
+#endif
 }
 
 /*
@@ -494,31 +538,20 @@ idCinematicLocal::~idCinematicLocal()
 	// Carl: ffmpeg for bink and other video files:
 
 	// RB: TODO double check this. It seems we have different versions of ffmpeg on Kubuntu 13.10 and the win32 development files
-#if defined(_WIN32) || defined(_WIN64)
+//#if defined(_WIN32) || defined(_WIN64)
+	// SRS - Should use the same version criteria as when the frames are allocated in idCinematicLocal() above
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
 	av_frame_free( &frame );
 	av_frame_free( &frame2 );
+	av_frame_free( &frame3 );
 #else
 	av_freep( &frame );
 	av_freep( &frame2 );
+	av_freep( &frame3 );
 #endif
-
-	if( fmt_ctx )
-	{
-		avformat_free_context( fmt_ctx );
-	}
-
-	if( img_convert_ctx )
-	{
-		sws_freeContext( img_convert_ctx );
-	}
 #endif
 
 #ifdef USE_BINKDEC
-	if( binkHandle.isValid )
-	{
-		Bink_Close( binkHandle );
-	}
-
 	delete imgY;
 	imgY = NULL;
 	delete imgCr;
@@ -529,6 +562,51 @@ idCinematicLocal::~idCinematicLocal()
 
 	delete img;
 	img = NULL;
+
+	//GK: Properly close local XAudio2 or OpenAL voice
+	cinematicAudio->ShutdownAudio();
+}
+
+#if defined(USE_FFMPEG)
+/*
+==============
+idCinematicLocal::GetSampleFormat
+==============
+*/
+const char* GetSampleFormat( AVSampleFormat sample_fmt )
+{
+	switch( sample_fmt )
+	{
+		case AV_SAMPLE_FMT_U8:
+		case AV_SAMPLE_FMT_U8P:
+		{
+			return "8-bit";
+		}
+		case AV_SAMPLE_FMT_S16:
+		case AV_SAMPLE_FMT_S16P:
+		{
+			return "16-bit";
+		}
+		case AV_SAMPLE_FMT_S32:
+		case AV_SAMPLE_FMT_S32P:
+		{
+			return "32-bit";
+		}
+		case AV_SAMPLE_FMT_FLT:
+		case AV_SAMPLE_FMT_FLTP:
+		{
+			return "Float";
+		}
+		case AV_SAMPLE_FMT_DBL:
+		case AV_SAMPLE_FMT_DBLP:
+		{
+			return "Double";
+		}
+		default:
+		{
+			return "Unknown";
+		}
+	}
 }
 
 /*
@@ -536,10 +614,11 @@ idCinematicLocal::~idCinematicLocal()
 idCinematicLocal::InitFromFFMPEGFile
 ==============
 */
-#if defined(USE_FFMPEG)
 bool idCinematicLocal::InitFromFFMPEGFile( const char* qpath, bool amilooping )
 {
 	int ret;
+	int ret2;
+	int file_size;
 	looping = amilooping;
 	startTime = 0;
 	isRoQ = false;
@@ -551,6 +630,7 @@ bool idCinematicLocal::InitFromFFMPEGFile( const char* qpath, bool amilooping )
 	if( testFile )
 	{
 		fullpath = testFile->GetFullPath();
+		file_size = testFile->Length();
 		fileSystem->CloseFile( testFile );
 	}
 	// RB: case sensitivity HACK for Linux
@@ -563,6 +643,7 @@ bool idCinematicLocal::InitFromFFMPEGFile( const char* qpath, bool amilooping )
 		if( testFile )
 		{
 			fullpath = testFile->GetFullPath();
+			file_size = testFile->Length();
 			fileSystem->CloseFile( testFile );
 		}
 		else
@@ -576,8 +657,20 @@ bool idCinematicLocal::InitFromFFMPEGFile( const char* qpath, bool amilooping )
 
 	if( ( ret = avformat_open_input( &fmt_ctx, fullpath, NULL, NULL ) ) < 0 )
 	{
-		common->Warning( "idCinematic: Cannot open FFMPEG video file: '%s', %d\n", qpath, looping );
-		return false;
+		// SRS - another case sensitivity hack for Linux, this time for ffmpeg and RoQ files
+		idStr ext;
+		fullpath.ExtractFileExtension( ext );
+		if( idStr::Cmp( ext.c_str(), "roq" ) == 0 )
+		{
+			// SRS - If ffmpeg can't open .roq file, then try again with .RoQ extension instead
+			fullpath.Replace( ".roq", ".RoQ" );
+			ret = avformat_open_input( &fmt_ctx, fullpath, NULL, NULL );
+		}
+		if( ret < 0 )
+		{
+			common->Warning( "idCinematic: Cannot open FFMPEG video file: '%s', %d\n", qpath, looping );
+			return false;
+		}
 	}
 	if( ( ret = avformat_find_stream_info( fmt_ctx, NULL ) ) < 0 )
 	{
@@ -592,14 +685,66 @@ bool idCinematicLocal::InitFromFFMPEGFile( const char* qpath, bool amilooping )
 		return false;
 	}
 	video_stream_index = ret;
-	dec_ctx = fmt_ctx->streams[video_stream_index]->codec;
+	dec_ctx = avcodec_alloc_context3( dec );
+	if( ( ret = avcodec_parameters_to_context( dec_ctx, fmt_ctx->streams[video_stream_index]->codecpar ) ) < 0 )
+	{
+		char* error = new char[256];
+		av_strerror( ret, error, 256 );
+		common->Warning( "idCinematic: Failed to create video codec context from codec parameters with error: %s\n", error );
+	}
+	dec_ctx->time_base = fmt_ctx->streams[video_stream_index]->time_base;
+	dec_ctx->framerate = fmt_ctx->streams[video_stream_index]->avg_frame_rate;
+	dec_ctx->pkt_timebase = fmt_ctx->streams[video_stream_index]->time_base;
 	/* init the video decoder */
 	if( ( ret = avcodec_open2( dec_ctx, dec, NULL ) ) < 0 )
 	{
-		common->Warning( "idCinematic: Cannot open video decoder for: '%s', %d\n", qpath, looping );
+		char* error = new char[256];
+		av_strerror( ret, error, 256 );
+		common->Warning( "idCinematic: Cannot open video decoder for: '%s', %d, with error: %s\n", qpath, looping, error );
 		return false;
 	}
-
+	//GK:Begin
+	//After the video decoder is open then try to open audio decoder
+	ret2 = av_find_best_stream( fmt_ctx, AVMEDIA_TYPE_AUDIO, -1, -1, &dec2, 0 );
+	if( ret2 >= 0 )  //Make audio optional (only intro video has audio no other)
+	{
+		audio_stream_index = ret2;
+		dec_ctx2 = avcodec_alloc_context3( dec2 );
+		if( ( ret2 = avcodec_parameters_to_context( dec_ctx2, fmt_ctx->streams[audio_stream_index]->codecpar ) ) < 0 )
+		{
+			char* error = new char[256];
+			av_strerror( ret2, error, 256 );
+			common->Warning( "idCinematic: Failed to create audio codec context from codec parameters with error: %s\n", error );
+		}
+		dec_ctx2->time_base = fmt_ctx->streams[audio_stream_index]->time_base;
+		dec_ctx2->framerate = fmt_ctx->streams[audio_stream_index]->avg_frame_rate;
+		dec_ctx2->pkt_timebase = fmt_ctx->streams[audio_stream_index]->time_base;
+		if( ( ret2 = avcodec_open2( dec_ctx2, dec2, NULL ) ) < 0 )
+		{
+			common->Warning( "idCinematic: Cannot open audio decoder for: '%s', %d\n", qpath, looping );
+			//return false;
+		}
+		if( dec_ctx2->sample_fmt >= AV_SAMPLE_FMT_U8P )											// SRS - Planar formats start at AV_SAMPLE_FMT_U8P
+		{
+			dst_smp = static_cast<AVSampleFormat>( dec_ctx2->sample_fmt - AV_SAMPLE_FMT_U8P );	// SRS - Setup context to convert from planar to packed
+			swr_ctx = swr_alloc_set_opts( NULL, dec_ctx2->channel_layout, dst_smp, dec_ctx2->sample_rate, dec_ctx2->channel_layout, dec_ctx2->sample_fmt, dec_ctx2->sample_rate, 0, NULL );
+			int res = swr_init( swr_ctx );
+			hasplanar = true;
+		}
+		else
+		{
+			dst_smp = dec_ctx2->sample_fmt;														// SRS - Must always define the destination format
+			hasplanar = false;
+		}
+		common->Printf( "Cinematic audio stream found: Sample Rate=%d Hz, Channels=%d, Format=%s, Planar=%d\n", dec_ctx2->sample_rate, dec_ctx2->channels, GetSampleFormat( dec_ctx2->sample_fmt ), hasplanar );
+		cinematicAudio->InitAudio( dec_ctx2 );
+	}
+	else
+	{
+		// SRS - Most cinematics have no audio, so disable the warning to reduce distracting log messages
+		//common->Warning("idCinematic: Cannot find an audio stream in: '%s', %d\n", qpath, looping);
+	}
+	//GK:End
 	CIN_WIDTH = dec_ctx->width;
 	CIN_HEIGHT = dec_ctx->height;
 	/** Calculate Duration in seconds
@@ -619,28 +764,43 @@ bool idCinematicLocal::InitFromFFMPEGFile( const char* qpath, bool amilooping )
 	 * Set to time_base ticks per frame. Default 1, e.g., H.264/MPEG-2 set it to 2.
 	 */
 	int ticksPerFrame = dec_ctx->ticks_per_frame;
-	float durationSec = static_cast<double>( fmt_ctx->streams[video_stream_index]->duration ) * static_cast<double>( ticksPerFrame ) / static_cast<double>( avr.den );
+	// SRS - In addition to ticks, must also use time_base numerator (not always 1) and denominator in the duration calculation
+	float durationSec = static_cast<double>( fmt_ctx->streams[video_stream_index]->duration ) * static_cast<double>( ticksPerFrame ) * static_cast<double>( avr.num ) / static_cast<double>( avr.den );
+	//GK: No duration is given. Check if we get at least bitrate to calculate the length, otherwise set it to a fixed 100 seconds (should it be lower ?)
+	if( durationSec < 0 )
+	{
+		// SRS - First check the file context bit rate and estimate duration using file size and overall bit rate
+		if( fmt_ctx->bit_rate > 0 )
+		{
+			durationSec = file_size * 8.0 / fmt_ctx->bit_rate;
+		}
+		// SRS - Likely an RoQ file, so use the video bit rate tolerance plus audio bit rate to estimate duration, then add 10% to correct for variable bit rate
+		else if( dec_ctx->bit_rate_tolerance > 0 )
+		{
+			durationSec = file_size * 8.0 / ( dec_ctx->bit_rate_tolerance + ( dec_ctx2 ? dec_ctx2->bit_rate : 0 ) ) * 1.1;
+		}
+		// SRS - Otherwise just set a large max duration
+		else
+		{
+			durationSec = 100.0;
+		}
+	}
 	animationLength = durationSec * 1000;
 	frameRate = av_q2d( fmt_ctx->streams[video_stream_index]->avg_frame_rate );
-	buf = NULL;
+	common->Printf( "Loaded FFMPEG file: '%s', looping=%d, %dx%d, %3.2f FPS, %4.1f sec\n", qpath, looping, CIN_WIDTH, CIN_HEIGHT, frameRate, durationSec );
+
+	// SRS - Get number of image bytes needed by querying with NULL first, then allocate image and fill with correct parameters
+	int img_bytes = av_image_fill_arrays( frame2->data, frame2->linesize, NULL, AV_PIX_FMT_BGR32, CIN_WIDTH, CIN_HEIGHT, 1 );
+	image = ( byte* )Mem_Alloc( img_bytes, TAG_CINEMATIC );
+	av_image_fill_arrays( frame2->data, frame2->linesize, image, AV_PIX_FMT_BGR32, CIN_WIDTH, CIN_HEIGHT, 1 ); //GK: Straight out of the FFMPEG source code
+	img_convert_ctx = sws_getContext( dec_ctx->width, dec_ctx->height, dec_ctx->pix_fmt, CIN_WIDTH, CIN_HEIGHT, AV_PIX_FMT_BGR32, SWS_BICUBIC, NULL, NULL, NULL );
+
+	status = FMV_PLAY;
 	hasFrame = false;
 	framePos = -1;
-	common->Printf( "Loaded FFMPEG file: '%s', looping=%d%dx%d, %f FPS, %f sec\n", qpath, looping, CIN_WIDTH, CIN_HEIGHT, frameRate, durationSec );
-	image = ( byte* )Mem_Alloc( CIN_WIDTH * CIN_HEIGHT * 4 * 2, TAG_CINEMATIC );
-	avpicture_fill( ( AVPicture* )frame2, image, AV_PIX_FMT_BGR32, CIN_WIDTH, CIN_HEIGHT );
-	if( img_convert_ctx )
-	{
-		sws_freeContext( img_convert_ctx );
-	}
-	img_convert_ctx = sws_getContext( dec_ctx->width, dec_ctx->height, dec_ctx->pix_fmt, CIN_WIDTH, CIN_HEIGHT, AV_PIX_FMT_BGR32, SWS_BICUBIC, NULL, NULL, NULL );
-	status = FMV_PLAY;
-
-	startTime = 0;
 	ImageForTime( 0 );
 	status = ( looping ) ? FMV_PLAY : FMV_IDLE;
 
-	//startTime = Sys_Milliseconds();
-
 	return true;
 }
 #endif
@@ -658,10 +818,36 @@ void idCinematicLocal::FFMPEGReset()
 
 	framePos = -1;
 
-	if( av_seek_frame( fmt_ctx, video_stream_index, 0, 0 ) >= 0 )
+	// SRS - If we have an ffmpeg audio context and are not looping, or skipLag is true, reset audio to release any stale buffers
+	if( dec_ctx2 && ( !( looping && status == FMV_EOF ) || skipLag ) )
+	{
+		cinematicAudio->ResetAudio();
+
+		for( int i = 0; i < NUM_LAG_FRAMES; i++ )
+		{
+			lagBufSize[ i ] = 0;
+			if( lagBuffer[ i ] )
+			{
+				av_freep( &lagBuffer[ i ] );
+			}
+		}
+	}
+
+	// SRS - For non-RoQ (i.e. bik) files, use standard frame seek to rewind the stream
+	if( dec_ctx->codec_id != AV_CODEC_ID_ROQ && av_seek_frame( fmt_ctx, video_stream_index, 0, 0 ) >= 0 )
 	{
 		status = FMV_LOOPED;
 	}
+	// SRS - Special handling for RoQ files: only byte seek works and ffmpeg RoQ decoder needs reset
+	else if( dec_ctx->codec_id == AV_CODEC_ID_ROQ && av_seek_frame( fmt_ctx, video_stream_index, 0, AVSEEK_FLAG_BYTE ) >= 0 )
+	{
+		// Close and reopen the ffmpeg RoQ codec without clearing the context - this seems to reset the decoder properly
+		avcodec_close( dec_ctx );
+		avcodec_open2( dec_ctx, dec, NULL );
+
+		status = FMV_LOOPED;
+	}
+	// SRS - Can't rewind the stream so we really are at EOF
 	else
 	{
 		status = FMV_EOF;
@@ -718,19 +904,29 @@ bool idCinematicLocal::InitFromBinkDecFile( const char* qpath, bool amilooping )
 		CIN_HEIGHT = h;
 	}
 
+	// SRS - Support Bink Audio for cinematic playback
+	audioTracks = Bink_GetNumAudioTracks( binkHandle );
+	if( audioTracks > 0 )
+	{
+		trackIndex = 0;														// SRS - Use the first audio track - is this reasonable?
+		binkInfo = Bink_GetAudioTrackDetails( binkHandle, trackIndex );
+		common->Printf( "Cinematic audio stream found: Sample Rate=%d Hz, Channels=%d, Format=16-bit\n", binkInfo.sampleRate, binkInfo.nChannels );
+		cinematicAudio->InitAudio( &binkInfo );
+	}
+
 	frameRate = Bink_GetFrameRate( binkHandle );
 	numFrames = Bink_GetNumFrames( binkHandle );
-	float durationSec = frameRate * numFrames;
-	animationLength = durationSec;
-	buf = NULL;
+	float durationSec = numFrames / frameRate;      // SRS - fixed Bink durationSec calculation
+	animationLength = durationSec * 1000;           // SRS - animationLength is in milliseconds
+	common->Printf( "Loaded BinkDec file: '%s', looping=%d, %dx%d, %3.2f FPS, %4.1f sec\n", qpath, looping, CIN_WIDTH, CIN_HEIGHT, frameRate, durationSec );
 
-	common->Printf( "Loaded BinkDec file: '%s', looping=%d%dx%d, %f FPS, %f sec\n", qpath, looping, CIN_WIDTH, CIN_HEIGHT, frameRate, durationSec );
+	memset( yuvBuffer, 0, sizeof( yuvBuffer ) );
 
 	status = FMV_PLAY;
-
-	startTime = Sys_Milliseconds();
-	memset( yuvBuffer, 0, sizeof( yuvBuffer ) );
+	hasFrame = false;                               // SRS - Implemented hasFrame for BinkDec behaviour consistency with FFMPEG
 	framePos = -1;
+	ImageForTime( 0 );                              // SRS - Was missing initial call to ImageForTime() - fixes validation errors when using Vulkan renderer
+	status = ( looping ) ? FMV_PLAY : FMV_IDLE;     // SRS - Update status based on looping flag
 
 	return true;
 }
@@ -738,6 +934,13 @@ bool idCinematicLocal::InitFromBinkDecFile( const char* qpath, bool amilooping )
 void idCinematicLocal::BinkDecReset()
 {
 	framePos = -1;
+
+	// SRS - If we have bink audio tracks, reset audio to release any stale buffers (even if looping)
+	if( audioTracks > 0 )
+	{
+		cinematicAudio->ResetAudio();
+	}
+
 	Bink_GotoFrame( binkHandle, 0 );
 	status = FMV_LOOPED;
 }
@@ -752,7 +955,8 @@ bool idCinematicLocal::InitFromFile( const char* qpath, bool amilooping )
 {
 	unsigned short RoQID;
 
-	Close();
+	// SRS - Don't need to call Close() here, all initialization is handled by constructor
+	//Close();
 
 	inMemory = 0;
 	animationLength = 100000;
@@ -767,32 +971,42 @@ bool idCinematicLocal::InitFromFile( const char* qpath, bool amilooping )
 		sprintf( fileName, "%s", qpath );
 	}
 	// Carl: Look for original Doom 3 RoQ files first:
-	idStr ext;
-	fileName.ExtractFileExtension( ext );
-	fileName = fileName.StripFileExtension();
-	fileName = fileName + ".roq";
-	//if (fileName == "video\\loadvideo.roq") {
-	//	fileName = "video\\idlogo.roq";
+	idStr temp = fileName.StripFileExtension() + ".roq";
+
+	// SRS - Cool legacy support, but leaving this disabled since it might break existing mods
+	//if( temp == "video\\loadvideo.roq" )
+	//{
+	//	temp = "video\\idlogo.roq";
 	//}
 
-	iFile = fileSystem->OpenFileRead( fileName );
+	iFile = fileSystem->OpenFileRead( temp );
 
-	// Carl: If the RoQ file doesn't exist, try using ffmpeg instead:
+	// Carl: If the RoQ file doesn't exist, try using bik file extension instead:
 	if( !iFile )
 	{
+		//idLib::Warning( "Original Doom 3 RoQ Cinematic not found: '%s'\n", temp.c_str() );
 #if defined(USE_FFMPEG)
-		//idLib::Warning( "Original Doom 3 RoQ Cinematic not found: '%s'\n", fileName.c_str() );
-		idStr temp = fileName.StripFileExtension() + ".bik";
+		temp = fileName.StripFileExtension() + ".bik";
+		skipLag = false;				// SRS - Enable lag buffer for ffmpeg bik decoder AV sync
+
+		// SRS - Support RoQ cinematic playback via ffmpeg decoder - better quality plus audio support
+	}
+	else
+	{
+		fileSystem->CloseFile( iFile );	// SRS - Close the RoQ file and let ffmpeg reopen it
+		iFile = NULL;
+		skipLag = true;					// SRS - Disable lag buffer for ffmpeg RoQ decoder AV sync
+	}
+	{
+		// SRS End
+
 		animationLength = 0;
-		hasFrame = false;
-		RoQShutdown();
 		fileName = temp;
 		//idLib::Warning( "New filename: '%s'\n", fileName.c_str() );
 		return InitFromFFMPEGFile( fileName.c_str(), amilooping );
 #elif defined(USE_BINKDEC)
-		idStr temp = fileName.StripFileExtension() + ".bik";
+		temp = fileName.StripFileExtension() + ".bik";
 		animationLength = 0;
-		RoQShutdown();
 		fileName = temp;
 		//idLib::Warning( "New filename: '%s'\n", fileName.c_str() );
 		return InitFromBinkDecFile( fileName.c_str(), amilooping );
@@ -803,6 +1017,7 @@ bool idCinematicLocal::InitFromFile( const char* qpath, bool amilooping )
 	}
 	// Carl: The rest of this function is for original Doom 3 RoQ files:
 	isRoQ = true;
+	fileName = temp;
 	ROQSize = iFile->Length();
 
 	looping = amilooping;
@@ -828,6 +1043,7 @@ bool idCinematicLocal::InitFromFile( const char* qpath, bool amilooping )
 		RoQ_init();
 		status = FMV_PLAY;
 		ImageForTime( 0 );
+		common->Printf( "Loaded RoQ file: '%s', looping=%d, %dx%d, %3.2f FPS\n", fileName.c_str(), looping, CIN_WIDTH, CIN_HEIGHT, frameRate );
 		status = ( looping ) ? FMV_PLAY : FMV_IDLE;
 		return true;
 	}
@@ -851,23 +1067,43 @@ void idCinematicLocal::Close()
 		status = FMV_EOF;
 	}
 
-	RoQShutdown();
-
+	if( isRoQ )
+	{
+		RoQShutdown();
+	}
 #if defined(USE_FFMPEG)
-	hasFrame = false;
-
-	if( !isRoQ )
+	else //if( !isRoQ )
 	{
 		if( img_convert_ctx )
 		{
 			sws_freeContext( img_convert_ctx );
+			img_convert_ctx = NULL;
 		}
 
-		img_convert_ctx = NULL;
+		// SRS - Free audio codec context, resample context, and any lagged audio buffers
+		if( dec_ctx2 )
+		{
+			avcodec_free_context( &dec_ctx2 );
+
+			// SRS - Free resample context if we were decoding planar audio
+			if( swr_ctx )
+			{
+				swr_free( &swr_ctx );
+			}
+
+			for( int i = 0; i < NUM_LAG_FRAMES; i++ )
+			{
+				lagBufSize[ i ] = 0;
+				if( lagBuffer[ i ] )
+				{
+					av_freep( &lagBuffer[ i ] );
+				}
+			}
+		}
 
 		if( dec_ctx )
 		{
-			avcodec_close( dec_ctx );
+			avcodec_free_context( &dec_ctx );
 		}
 
 		if( fmt_ctx )
@@ -876,12 +1112,14 @@ void idCinematicLocal::Close()
 		}
 		status = FMV_EOF;
 	}
-#endif
-#ifdef USE_BINKDEC
-	if( !isRoQ && binkHandle.isValid )
+#elif defined(USE_BINKDEC)
+	else //if( !isRoQ )
 	{
-		memset( yuvBuffer, 0 , sizeof( yuvBuffer ) );
-		Bink_Close( binkHandle );
+		if( binkHandle.isValid )
+		{
+			memset( yuvBuffer, 0 , sizeof( yuvBuffer ) );
+			Bink_Close( binkHandle );
+		}
 		status = FMV_EOF;
 	}
 #endif
@@ -904,6 +1142,18 @@ bool idCinematicLocal::IsPlaying() const
 }
 // RB end
 
+// SRS - Implement virtual method to override abstract virtual method
+/*
+==============
+ idCinematicLocal::GetStartTime
+==============
+*/
+int idCinematicLocal::GetStartTime()
+{
+	return startTime;
+}
+// SRS end
+
 /*
 ==============
 idCinematicLocal::ResetTime
@@ -928,8 +1178,7 @@ cinData_t idCinematicLocal::ImageForTime( int thisTime )
 	{
 		return ImageForTimeFFMPEG( thisTime );
 	}
-#endif
-#ifdef USE_BINKDEC // DG: libbinkdec support
+#elif defined(USE_BINKDEC) // DG: libbinkdec support
 	if( !isRoQ )
 	{
 		return ImageForTimeBinkDec( thisTime );
@@ -939,15 +1188,16 @@ cinData_t idCinematicLocal::ImageForTime( int thisTime )
 	// Carl: Handle original Doom 3 RoQ video files
 	cinData_t	cinData;
 
-	if( thisTime == 0 )
+	// SRS - Changed from == 0 to <= 0 to match behaviour of FFMPEG and BinkDec decoders
+	if( thisTime <= 0 )
 	{
 		thisTime = Sys_Milliseconds();
 	}
 
-	if( thisTime < 0 )
-	{
-		thisTime = 0;
-	}
+	//if( thisTime < 0 )
+	//{
+	//	thisTime = 0;
+	//}
 
 	memset( &cinData, 0, sizeof( cinData ) );
 
@@ -984,7 +1234,8 @@ cinData_t idCinematicLocal::ImageForTime( int thisTime )
 		tfps = 0;
 	}
 
-	if( tfps < numQuads )
+	// SRS - Need to use numQuads - 1 for frame position (otherwise get into reset loop at start)
+	if( tfps < numQuads - 1 )
 	{
 		RoQReset();
 		buf = NULL;
@@ -993,6 +1244,7 @@ cinData_t idCinematicLocal::ImageForTime( int thisTime )
 
 	if( buf == NULL )
 	{
+		// SRS - This frame init loop is not really necessary, but leaving in to avoid breakage
 		while( buf == NULL )
 		{
 			RoQInterrupt();
@@ -1000,32 +1252,31 @@ cinData_t idCinematicLocal::ImageForTime( int thisTime )
 	}
 	else
 	{
-		while( ( tfps != numQuads && status == FMV_PLAY ) )
+		// SRS - This frame loop is really all we need and could handle the above case as well
+		while( ( numQuads - 1 < tfps && status == FMV_PLAY ) )
 		{
 			RoQInterrupt();
 		}
 	}
 
-	if( status == FMV_LOOPED )
-	{
-		status = FMV_PLAY;
-		while( buf == NULL && status == FMV_PLAY )
-		{
-			RoQInterrupt();
-		}
-		startTime = thisTime;
-	}
+	// SRS - This is redundant code, virtually identical logic correctly handles looping below
+	//if( status == FMV_LOOPED )
+	//{
+	//	status = FMV_PLAY;
+	//	while( buf == NULL && status == FMV_PLAY )
+	//	{
+	//		RoQInterrupt();
+	//	}
+	//	startTime = thisTime;
+	//}
 
-	if( status == FMV_EOF )
+	if( status == FMV_LOOPED || status == FMV_EOF )
 	{
 		if( looping )
 		{
-			RoQReset();
+			//RoQReset();		// SRS - RoQReset() already called by RoQInterrupt() when looping
 			buf = NULL;
-			if( status == FMV_LOOPED )
-			{
-				status = FMV_PLAY;
-			}
+			status = FMV_PLAY;
 			while( buf == NULL && status == FMV_PLAY )
 			{
 				RoQInterrupt();
@@ -1034,8 +1285,10 @@ cinData_t idCinematicLocal::ImageForTime( int thisTime )
 		}
 		else
 		{
+			buf = NULL;
 			status = FMV_IDLE;
-			RoQShutdown();
+			//RoQShutdown();	//SRS - RoQShutdown() not needed on EOF, return null data instead
+			return cinData;
 		}
 	}
 
@@ -1057,6 +1310,8 @@ idCinematicLocal::ImageForTimeFFMPEG
 cinData_t idCinematicLocal::ImageForTimeFFMPEG( int thisTime )
 {
 	cinData_t	cinData;
+	uint8_t*	audioBuffer = NULL;
+	int			num_bytes = 0;
 
 	if( thisTime <= 0 )
 	{
@@ -1093,6 +1348,8 @@ cinData_t idCinematicLocal::ImageForTimeFFMPEG( int thisTime )
 	if( desiredFrame < framePos )
 	{
 		FFMPEGReset();
+		hasFrame = false;
+		status = FMV_PLAY;
 	}
 
 	if( hasFrame && desiredFrame == framePos )
@@ -1107,10 +1364,14 @@ cinData_t idCinematicLocal::ImageForTimeFFMPEG( int thisTime )
 	AVPacket packet;
 	while( framePos < desiredFrame )
 	{
-		int frameFinished = 0;
+		int frameFinished = -1;
+		//GK: Separate frame finisher for audio in order to not have the video lagging
+		int frameFinished1 = -1;
+
+		int res = 0;
 
 		// Do a single frame by getting packets until we have a full frame
-		while( !frameFinished )
+		while( frameFinished != 0 )
 		{
 			// if we got to the end or failed
 			if( av_read_frame( fmt_ctx, &packet ) < 0 )
@@ -1121,7 +1382,7 @@ cinData_t idCinematicLocal::ImageForTimeFFMPEG( int thisTime )
 				{
 					desiredFrame = 0;
 					FFMPEGReset();
-					framePos = -1;
+					hasFrame = false;
 					startTime = thisTime;
 					if( av_read_frame( fmt_ctx, &packet ) < 0 )
 					{
@@ -1132,6 +1393,7 @@ cinData_t idCinematicLocal::ImageForTimeFFMPEG( int thisTime )
 				}
 				else
 				{
+					hasFrame = false;
 					status = FMV_IDLE;
 					return cinData;
 				}
@@ -1140,10 +1402,71 @@ cinData_t idCinematicLocal::ImageForTimeFFMPEG( int thisTime )
 			if( packet.stream_index == video_stream_index )
 			{
 				// Decode video frame
-				avcodec_decode_video2( dec_ctx, frame, &frameFinished, &packet );
+				if( ( res = avcodec_send_packet( dec_ctx, &packet ) ) != 0 )
+				{
+					char* error = new char[256];
+					av_strerror( res, error, 256 );
+					common->Warning( "idCinematic: Failed to send video packet for decoding with error: %s\n", error );
+				}
+				else
+				{
+					if( ( frameFinished = avcodec_receive_frame( dec_ctx, frame ) ) != 0 )
+					{
+						char* error = new char[256];
+						av_strerror( frameFinished, error, 256 );
+						common->Warning( "idCinematic: Failed to receive video frame from decoding with error: %s\n", error );
+					}
+				}
 			}
+			//GK:Begin
+			else if( packet.stream_index == audio_stream_index ) //Check if it found any audio data
+			{
+				res = avcodec_send_packet( dec_ctx2, &packet );
+				if( res != 0 && res != AVERROR( EAGAIN ) )
+				{
+					char* error = new char[256];
+					av_strerror( res, error, 256 );
+					common->Warning( "idCinematic: Failed to send audio packet for decoding with error: %s\n", error );
+				}
+				else
+				{
+					if( ( frameFinished1 = avcodec_receive_frame( dec_ctx2, frame3 ) ) != 0 )
+					{
+						char* error = new char[256];
+						av_strerror( frameFinished1, error, 256 );
+						common->Warning( "idCinematic: Failed to receive audio frame from decoding with error: %s\n", error );
+					}
+					else
+					{
+						// SRS - Since destination sample format is packed (non-planar), returned bufflinesize equals num_bytes
+						res = av_samples_alloc( &audioBuffer, &num_bytes, frame3->channels, frame3->nb_samples, dst_smp, 0 );
+						if( res < 0 || res != num_bytes )
+						{
+							common->Warning( "idCinematic: Failed to allocate audio buffer with result: %d\n", res );
+						}
+						if( hasplanar )
+						{
+							// SRS - Convert from planar to packed format keeping sample count the same
+							res = swr_convert( swr_ctx, &audioBuffer, frame3->nb_samples, ( const uint8_t** )frame3->extended_data, frame3->nb_samples );
+							if( res < 0 || res != frame3->nb_samples )
+							{
+								common->Warning( "idCinematic: Failed to convert planar audio data to packed format with result: %d\n", res );
+							}
+						}
+						else
+						{
+							// SRS - Since audio is already in packed format, just copy into audio buffer
+							if( num_bytes > 0 )
+							{
+								memcpy( audioBuffer, frame3->extended_data[0], num_bytes );
+							}
+						}
+					}
+				}
+			}
+			//GK:End
 			// Free the packet that was allocated by av_read_frame
-			av_free_packet( &packet );
+			av_packet_unref( &packet );
 		}
 
 		framePos++;
@@ -1159,6 +1482,23 @@ cinData_t idCinematicLocal::ImageForTimeFFMPEG( int thisTime )
 	hasFrame = true;
 	cinData.image = img;
 
+	// SRS - If we have cinematic audio data, play a lagged frame (for FFMPEG video sync) and save the current frame
+	if( num_bytes > 0 )
+	{
+		// SRS - If we have a lagged cinematic audio frame, then play it now
+		if( lagBufSize[ lagIndex ] > 0 )
+		{
+			// SRS - Note that PlayAudio() is responsible for releasing any audio buffers sent to it
+			cinematicAudio->PlayAudio( lagBuffer[ lagIndex ], lagBufSize[ lagIndex ] );
+		}
+
+		// SRS - Save the current (new) audio buffer and its size to play NUM_LAG_FRAMES in the future
+		lagBuffer[ lagIndex ] = audioBuffer;
+		lagBufSize[ lagIndex ] = num_bytes;
+
+		lagIndex = ( lagIndex + 1 ) % ( skipLag ? 1 : NUM_LAG_FRAMES );
+	}
+
 	return cinData;
 }
 #endif
@@ -1167,13 +1507,16 @@ cinData_t idCinematicLocal::ImageForTimeFFMPEG( int thisTime )
 #ifdef USE_BINKDEC
 cinData_t idCinematicLocal::ImageForTimeBinkDec( int thisTime )
 {
-	cinData_t	cinData = {0};
+	cinData_t	cinData;
+	int16_t*	audioBuffer = NULL;
+	uint32_t	num_bytes = 0;
 
 	if( thisTime <= 0 )
 	{
 		thisTime = Sys_Milliseconds();
 	}
 
+	memset( &cinData, 0, sizeof( cinData ) );
 	if( r_skipDynamicTextures.GetBool() || status == FMV_EOF || status == FMV_IDLE )
 	{
 		return cinData;
@@ -1185,9 +1528,13 @@ cinData_t idCinematicLocal::ImageForTimeBinkDec( int thisTime )
 		return cinData;
 	}
 
-	if( startTime == -1 )
+	// SRS - Implement hasFrame so BinkDec startTime is handled the same as with FFMPEG
+	if( ( !hasFrame ) || startTime == -1 )
 	{
-		BinkDecReset();
+		if( startTime == -1 )
+		{
+			BinkDecReset();
+		}
 		startTime = thisTime;
 	}
 
@@ -1197,25 +1544,16 @@ cinData_t idCinematicLocal::ImageForTimeBinkDec( int thisTime )
 		desiredFrame = 0;
 	}
 
-	if( desiredFrame >= numFrames )
+	// SRS - Enable video replay within PDAs
+	if( desiredFrame < framePos )
 	{
-		status = FMV_EOF;
-		if( looping )
-		{
-			desiredFrame = 0;
-			BinkDecReset();
-			framePos = -1;
-			startTime = thisTime;
-			status = FMV_PLAY;
-		}
-		else
-		{
-			status = FMV_IDLE;
-			return cinData;
-		}
+		BinkDecReset();
+		hasFrame = false;
+		status = FMV_PLAY;
 	}
+	// SRS end
 
-	if( desiredFrame == framePos )
+	if( hasFrame && desiredFrame == framePos )
 	{
 		cinData.imageWidth = CIN_WIDTH;
 		cinData.imageHeight = CIN_HEIGHT;
@@ -1227,6 +1565,25 @@ cinData_t idCinematicLocal::ImageForTimeBinkDec( int thisTime )
 		return cinData;
 	}
 
+	if( desiredFrame >= numFrames )
+	{
+		status = FMV_EOF;
+		if( looping )
+		{
+			desiredFrame = 0;
+			BinkDecReset();
+			hasFrame = false;
+			startTime = thisTime;
+			status = FMV_PLAY;
+		}
+		else
+		{
+			hasFrame = false;
+			status = FMV_IDLE;
+			return cinData;
+		}
+	}
+
 	// Bink_GotoFrame(binkHandle, desiredFrame);
 	// apparently Bink_GotoFrame() doesn't work super well, so skip frames
 	// (if necessary) by calling Bink_GetNextFrame()
@@ -1259,6 +1616,11 @@ cinData_t idCinematicLocal::ImageForTimeBinkDec( int thisTime )
 		}
 		else if( h < CIN_HEIGHT )
 		{
+#if defined(__APPLE__) && defined(USE_VULKAN)
+			// SRS - For U and V channels on OSX Vulkan use full height image to work around stall that occurs with half-height chroma planes
+			// when exiting levels or returning from demo playback - depends on OSX-specific logic inside Vulkan version of SubImageUpload()
+			h = CIN_HEIGHT;
+#else
 			// the U and V channels have a lower resolution than the Y channel
 			// (or the logical video resolution), so use the aspect ratio to
 			// calculate the real height
@@ -1267,6 +1629,7 @@ cinData_t idCinematicLocal::ImageForTimeBinkDec( int thisTime )
 			{
 				h = hExp;
 			}
+#endif
 		}
 
 		if( img->GetUploadWidth() != w || img->GetUploadHeight() != h )
@@ -1279,10 +1642,29 @@ cinData_t idCinematicLocal::ImageForTimeBinkDec( int thisTime )
 		img->SubImageUpload( 0, 0, 0, 0, w, h, yuvBuffer[i].data );
 	}
 
+	hasFrame = true;
 	cinData.imageY = imgY;
 	cinData.imageCr = imgCr;
 	cinData.imageCb = imgCb;
 
+	if( audioTracks > 0 )
+	{
+		audioBuffer = ( int16_t* )Mem_Alloc( binkInfo.idealBufferSize, TAG_AUDIO );
+		num_bytes = Bink_GetAudioData( binkHandle, trackIndex, audioBuffer );
+
+		// SRS - If we have cinematic audio data, start playing it now
+		if( num_bytes > 0 )
+		{
+			// SRS - Note that PlayAudio() is responsible for releasing any audio buffers sent to it
+			cinematicAudio->PlayAudio( ( uint8_t* )audioBuffer, num_bytes );
+		}
+		else
+		{
+			// SRS - Even though we have no audio data to play, still need to free the audio buffer
+			Mem_Free( audioBuffer );
+		}
+	}
+
 	return cinData;
 }
 #endif
@@ -2758,7 +3140,7 @@ redump:
 //
 // read in next frame data
 //
-	if( RoQPlayed >= ROQSize )
+	if( RoQPlayed >= ROQSize || status == FMV_EOF )		// SRS - handle FMV_EOF case
 	{
 		if( looping )
 		{
@@ -2834,11 +3216,14 @@ idCinematicLocal::RoQShutdown
 */
 void idCinematicLocal::RoQShutdown()
 {
+	// SRS - Depending on status, this could prevent closing of iFile on shutdown, disable it
+	/*
 	if( status == FMV_IDLE )
 	{
 		return;
 	}
-	status = FMV_IDLE;
+	*/
+	status = FMV_EOF;	// SRS - Changed from FMV_IDLE to FMV_EOF for shutdown consistency
 
 	if( iFile )
 	{
