@@ -5,6 +5,7 @@ Doom 3 BFG Edition GPL Source Code
 Copyright (C) 1993-2012 id Software LLC, a ZeniMax Media company.
 Copyright (C) 2013-2021 Robert Beckebans
 Copyright (C) 2016-2017 Dustin Land
+Copyright (C) 2022 Stephen Pridham
 
 This file is part of the Doom 3 BFG Edition GPL Source Code ("Doom 3 BFG Edition Source Code").
 
@@ -28,6 +29,9 @@ If you have questions concerning this license or the applicable additional terms
 ===========================================================================
 */
 
+#ifndef IMAGE_H_
+#define IMAGE_H_
+
 enum textureType_t
 {
 	TT_DISABLED,
@@ -35,7 +39,7 @@ enum textureType_t
 	TT_CUBIC,
 	// RB begin
 	TT_2D_ARRAY,
-	TT_2D_MULTISAMPLE
+	TT_2D_MULTISAMPLE,
 	// RB end
 };
 
@@ -101,6 +105,8 @@ enum textureFormat_t
 	FMT_RGBA32F,		// 128 bpp
 	FMT_R32F,			// 32 bpp
 	FMT_R11G11B10F,		// 32 bpp
+	FMT_R8,
+	FMT_DEPTH_STENCIL,  // 32 bpp
 	// RB end
 };
 
@@ -157,6 +163,7 @@ public:
 	int					numLevels;		// if 0, will be 1 for NEAREST / LINEAR filters, otherwise based on size
 	bool				gammaMips;		// if true, mips will be generated with gamma correction
 	bool				readback;		// 360 specific - cpu reads back from this texture, so allocate with cached memory
+	bool				isRenderTarget;
 };
 
 /*
@@ -175,8 +182,8 @@ ID_INLINE idImageOpts::idImageOpts()
 	textureType		= TT_2D;
 	gammaMips		= false;
 	readback		= false;
-
-};
+	isRenderTarget	= false;
+}
 
 /*
 ========================
@@ -229,6 +236,8 @@ typedef enum
 	TD_R32F,
 	TD_R11G11B10F,			// memory efficient HDR RGB format with only 32bpp
 	// RB end
+	TD_R8F,					// Stephen: Added for ambient occlusion render target.
+	TD_DEPTH_STENCIL,       // depth buffer and stencil buffer
 } textureUsage_t;
 
 typedef enum
@@ -238,7 +247,8 @@ typedef enum
 	CF_CAMERA,		// _forward, _back, etc, rotated and flipped as needed before sending to GL
 	CF_PANORAMA,	// TODO latlong encoded HDRI panorama typically used by Substance or Blender
 	CF_2D_ARRAY,	// not a cube map but not a single 2d texture either
-	CF_2D_PACKED_MIPCHAIN // usually 2d but can be an octahedron, packed mipmaps into single 2d texture atlas and limited to dim^2
+	CF_2D_PACKED_MIPCHAIN, // usually 2d but can be an octahedron, packed mipmaps into single 2d texture atlas and limited to dim^2
+	CF_SINGLE,      // SP: A single texture cubemap. All six sides in one image.
 } cubeFiles_t;
 
 enum imageFileType_t
@@ -266,7 +276,7 @@ public:
 		return imgName;
 	}
 
-	// Makes this image active on the current GL texture unit.
+	// Makes this image active on the current texture unit.
 	// automatically enables or disables cube mapping
 	// May perform file loading if the image was not preloaded.
 	void		Bind();
@@ -365,12 +375,6 @@ public:
 								int width, int height, const void* data,
 								int pixelPitch = 0 );
 
-	// SetPixel is assumed to be a fast memory write on consoles, degenerating to a
-	// SubImageUpload on PCs.  Used to update the page mapping images.
-	// We could remove this now, because the consoles don't use the intermediate page mapping
-	// textures now that they can pack everything into the virtual page table images.
-	void		SetPixel( int mipLevel, int x, int y, const void* data, int dataSize );
-
 	// some scratch images are dynamically resized based on the display window size.  This
 	// simply purges the image and recreates it if the sizes are different, so it should not be
 	// done under any normal circumstances, and probably not at all on consoles.
@@ -405,7 +409,9 @@ public:
 							   textureFilter_t filter,
 							   textureRepeat_t repeat,
 							   textureUsage_t usage,
-							   textureSamples_t samples = SAMPLE_1, cubeFiles_t cubeFiles = CF_2D );
+							   textureSamples_t samples = SAMPLE_1,
+							   cubeFiles_t cubeFiles = CF_2D,
+							   bool isRenderTarget = false );
 
 	void		GenerateCubeImage( const byte* pic[6], int size,
 								   textureFilter_t filter, textureUsage_t usage );
@@ -433,13 +439,14 @@ public:
 private:
 	friend class idImageManager;
 
-	void		DeriveOpts();
-	void		AllocImage();
-	void		SetSamplerState( textureFilter_t tf, textureRepeat_t tr );
+	void				DeriveOpts();
+	void				AllocImage();
+	void				SetSamplerState( textureFilter_t tf, textureRepeat_t tr );
 
 	// parameters that define this image
 	idStr				imgName;				// game path, including extension (except for cube maps), may be an image program
 	cubeFiles_t			cubeFiles;				// If this is a cube map, and if so, what kind
+	int                 cubeMapSize;
 	void	( *generatorFunction )( idImage* image );	// NULL for files
 	textureUsage_t		usage;					// Used to determine the type of compression to use
 	idImageOpts			opts;					// Parameters that determine the storage method
@@ -494,6 +501,9 @@ private:
 };
 
 // data is RGBA
+void	LoadPNG( const char* filename, unsigned char** pic, int* width, int* height, ID_TIME_T* timestamp );
+
+void	LoadTGA( const char* name, byte** pic, int* width, int* height, ID_TIME_T* timestamp );
 void	R_WriteTGA( const char* filename, const byte* data, int width, int height, bool flipVertical = false, const char* basePath = "fs_savepath" );
 // data is in top-to-bottom raster order unless flipVertical is set
 
@@ -523,7 +533,7 @@ public:
 	// grid pattern.
 	// Will automatically execute image programs if needed.
 	idImage* 			ImageFromFile( const char* name,
-									   textureFilter_t filter, textureRepeat_t repeat, textureUsage_t usage, cubeFiles_t cubeMap = CF_2D );
+									   textureFilter_t filter, textureRepeat_t repeat, textureUsage_t usage, cubeFiles_t cubeMap = CF_2D, int cubeMapSize = 0 );
 
 	// look for a loaded image, whatever the parameters
 	idImage* 			GetImage( const char* name ) const;
@@ -653,6 +663,11 @@ void R_VerticalFlip( byte* data, int width, int height );
 void R_VerticalFlipRGB16F( byte* data, int width, int height );
 void R_RotatePic( byte* data, int width );
 void R_ApplyCubeMapTransforms( int i, byte* data, int size );
+// SP begin
+// This method takes in a cubemap from a single image. Depending on the side (0-5),
+// the image will be extracted from data and returned. The dimensions will be size x size.
+byte* R_GenerateCubeMapSideFromSingleImage( byte* data, int srcWidth, int srcHeight, int size, int side );
+// SP end
 
 idVec4 R_CalculateMipRect( uint dimensions, uint mip );
 int R_CalculateUsedAtlasPixels( int dimensions );
@@ -669,7 +684,7 @@ IMAGEFILES
 void R_LoadImage( const char* name, byte** pic, int* width, int* height, ID_TIME_T* timestamp, bool makePowerOf2, textureUsage_t* usage );
 
 // pic is in top to bottom raster format
-bool R_LoadCubeImages( const char* cname, cubeFiles_t extensions, byte* pic[6], int* size, ID_TIME_T* timestamp );
+bool R_LoadCubeImages( const char* cname, cubeFiles_t extensions, byte* pic[6], int* size, ID_TIME_T* timestamp, int cubeMapSize = 0 );
 
 /*
 ====================================================================
@@ -682,3 +697,4 @@ IMAGEPROGRAM
 void R_LoadImageProgram( const char* name, byte** pic, int* width, int* height, ID_TIME_T* timestamp, textureUsage_t* usage = NULL );
 const char* R_ParsePastImageProgram( idLexer& src );
 
+#endif
\ No newline at end of file
