@@ -82,10 +82,14 @@ int BitsForFormat( textureFormat_t format )
 		// RB end
 		case FMT_DEPTH:
 			return 32;
+		case FMT_DEPTH_STENCIL:
+			return 32;
 		case FMT_X16:
 			return 16;
 		case FMT_Y16_X16:
 			return 32;
+		case FMT_R8:
+			return 4;
 		default:
 			assert( 0 );
 			return 0;
@@ -114,6 +118,12 @@ ID_INLINE void idImage::DeriveOpts()
 				opts.format = FMT_DEPTH;
 				break;
 
+			// sp begin
+			case TD_DEPTH_STENCIL:
+				opts.format = FMT_DEPTH_STENCIL;
+				break;
+			// sp end
+
 			case TD_SHADOW_ARRAY:
 				opts.format = FMT_SHADOW_ARRAY;
 				break;
@@ -134,6 +144,10 @@ ID_INLINE void idImage::DeriveOpts()
 				opts.format = FMT_R32F;
 				break;
 
+			case TD_R8F:
+				opts.format = FMT_R8;
+				break;
+
 			case TD_R11G11B10F:
 				opts.format = FMT_R11G11B10F;
 				break;
@@ -273,7 +287,6 @@ void idImage::GetGeneratedName( idStr& _name, const textureUsage_t& _usage, cons
 	}
 }
 
-
 /*
 ===============
 ActuallyLoadImage
@@ -319,11 +332,11 @@ void idImage::ActuallyLoadImage( bool fromBackEnd )
 		{
 			opts.textureType = TT_2D_ARRAY;
 		}
-		else if( cubeFiles == CF_NATIVE || cubeFiles == CF_CAMERA )
+		else if( cubeFiles == CF_NATIVE || cubeFiles == CF_CAMERA || cubeFiles == CF_SINGLE )
 		{
 			opts.textureType = TT_CUBIC;
 			repeat = TR_CLAMP;
-			R_LoadCubeImages( GetName(), cubeFiles, NULL, NULL, &sourceFileTime );
+			R_LoadCubeImages( GetName(), cubeFiles, NULL, NULL, &sourceFileTime, cubeMapSize );
 		}
 		else
 		{
@@ -423,7 +436,9 @@ void idImage::ActuallyLoadImage( bool fromBackEnd )
 		{
 			opts.format = ( textureFormat_t )header.format;
 		}
+
 		opts.textureType = ( textureType_t )header.textureType;
+
 		if( cvarSystem->GetCVarBool( "fs_buildresources" ) )
 		{
 			// for resource gathering write this image to the preload file for this map
@@ -454,12 +469,12 @@ void idImage::ActuallyLoadImage( bool fromBackEnd )
 		//else if( toolUsage )
 		//	binarizeReason = va( "binarize: tool usage '%s'", generatedName.c_str() );
 
-		if( cubeFiles == CF_NATIVE || cubeFiles == CF_CAMERA )
+		if( cubeFiles == CF_NATIVE || cubeFiles == CF_CAMERA || cubeFiles == CF_SINGLE )
 		{
 			int size;
 			byte* pics[6];
 
-			if( !R_LoadCubeImages( GetName(), cubeFiles, pics, &size, &sourceFileTime ) || size == 0 )
+			if( !R_LoadCubeImages( GetName(), cubeFiles, pics, &size, &sourceFileTime, cubeMapSize ) || size == 0 )
 			{
 				idLib::Warning( "Couldn't load cube image: %s", GetName() );
 				defaulted = true; // RB
@@ -738,11 +753,20 @@ idImage::Reload
 */
 void idImage::Reload( bool force )
 {
+	// don't break render targets that have this image attached
+	if( opts.isRenderTarget )
+	{
+		return;
+	}
+
 	// always regenerate functional images
 	if( generatorFunction )
 	{
-		common->DPrintf( "regenerating %s.\n", GetName() );
-		generatorFunction( this );
+		if( force )
+		{
+			common->DPrintf( "regenerating %s.\n", GetName() );
+			generatorFunction( this );
+		}
 		return;
 	}
 
@@ -750,7 +774,7 @@ void idImage::Reload( bool force )
 	if( !force )
 	{
 		ID_TIME_T current;
-		if( cubeFiles == CF_NATIVE || cubeFiles == CF_CAMERA )
+		if( cubeFiles == CF_NATIVE || cubeFiles == CF_CAMERA || cubeFiles == CF_SINGLE )
 		{
 			R_LoadCubeImages( imgName, cubeFiles, NULL, NULL, &current );
 		}
@@ -778,7 +802,7 @@ void idImage::Reload( bool force )
 GenerateImage
 ================
 */
-void idImage::GenerateImage( const byte* pic, int width, int height, textureFilter_t filterParm, textureRepeat_t repeatParm, textureUsage_t usageParm, textureSamples_t samples, cubeFiles_t _cubeFiles )
+void idImage::GenerateImage( const byte* pic, int width, int height, textureFilter_t filterParm, textureRepeat_t repeatParm, textureUsage_t usageParm, textureSamples_t samples, cubeFiles_t _cubeFiles, bool isRenderTarget )
 {
 	PurgeImage();
 
@@ -792,6 +816,7 @@ void idImage::GenerateImage( const byte* pic, int width, int height, textureFilt
 	opts.height = height;
 	opts.numLevels = 0;
 	opts.samples = samples;
+	opts.isRenderTarget = isRenderTarget;
 
 	// RB
 	if( cubeFiles == CF_2D_PACKED_MIPCHAIN )
@@ -940,6 +965,8 @@ void idImage::GenerateShadowArray( int width, int height, textureFilter_t filter
 	opts.width = width;
 	opts.height = height;
 	opts.numLevels = 0;
+	opts.isRenderTarget = true;
+
 	DeriveOpts();
 
 	// if we don't have a rendering context, just return after we
@@ -980,6 +1007,7 @@ void idImage::UploadScratch( const byte* data, int cols, int rows )
 	{
 		rows /= 6;
 		const byte* pic[6];
+
 		for( int i = 0; i < 6; i++ )
 		{
 			pic[i] = data + cols * rows * 4 * i;
@@ -990,6 +1018,7 @@ void idImage::UploadScratch( const byte* data, int cols, int rows )
 			GenerateCubeImage( pic, cols, TF_LINEAR, TD_LOOKUP_TABLE_RGBA );
 			return;
 		}
+
 		if( opts.width != cols || opts.height != rows )
 		{
 			opts.width = cols;
