@@ -4,6 +4,7 @@
 Doom 3 BFG Edition GPL Source Code
 Copyright (C) 1993-2012 id Software LLC, a ZeniMax Media company.
 Copyright (C) 2021 Robert Beckebans
+Copyright (C) 2021 Stephen Pridham
 
 This file is part of the Doom 3 BFG Edition GPL Source Code ("Doom 3 BFG Edition Source Code").
 
@@ -628,3 +629,77 @@ int R_CalculateUsedAtlasPixels( int dimensions )
 
 	return numPixels;
 }
+
+// SP begin
+
+byte* R_GenerateCubeMapSideFromSingleImage( byte* data, int srcWidth, int srcHeight, int size, int side )
+{
+	size_t x = 0, y = 0;
+	switch( side )
+	{
+		case 0:
+		{
+			// negative Z, front
+			x = size;
+			y = size;
+			break;
+		}
+		case 1:
+		{
+			// positive Z, back
+			x = 3 * size;
+			y = size;
+			break;
+		}
+		case 2:
+		{
+			// negative X, left
+			x = 0;
+			y = size;
+			break;
+		}
+		case 3:
+		{
+			// positive X, right
+			x = size * 2;
+			y = size;
+			break;
+		}
+		case 4:
+		{
+			// positive Y, top
+			x = size;
+			y = 0;
+			break;
+		}
+		case 5:
+		{
+			// negative Y, bottom
+			x = size;
+			y = 2 * ( size_t )size;
+			break;
+		}
+		default:
+		{
+			common->Warning( "Invalid side when generating cube map images" );
+			return nullptr;
+		}
+	}
+
+	const size_t copySize = ( size_t )size * ( size_t )size * 4;
+	byte* out = ( byte* )R_StaticAlloc( copySize, TAG_IMAGE );
+	uint32_t* out_p = ( uint32_t* )out;
+	const uint32_t* in_p = ( uint32_t* )data + x + y * srcWidth;
+
+	for( int j = 0; j < size; j++ )
+	{
+		for( int i = 0; i < size; i++ )
+		{
+			out_p[i + j * size] = in_p[i + j * srcWidth];
+		}
+	}
+
+	return out;
+}
+
+// SP end
