@@ -5,6 +5,7 @@ Doom 3 BFG Edition GPL Source Code
 Copyright (C) 1993-2012 id Software LLC, a ZeniMax Media company.
 Copyright (C) 2014-2020 Robert Beckebans
 Copyright (C) 2014-2016 Kot in Action Creative Artel
+Copyright (C) 2022 Stephen Pridham
 
 This file is part of the Doom 3 BFG Edition GPL Source Code ("Doom 3 BFG Edition Source Code").
 
@@ -97,7 +98,9 @@ typedef enum
 	DI_CUBE_RENDER,
 	DI_MIRROR_RENDER,
 	DI_XRAY_RENDER,
-	DI_REMOTE_RENDER
+	DI_REMOTE_RENDER,
+	DI_GUI_RENDER,
+	DI_RENDER_TARGET,
 } dynamicidImage_t;
 
 // note: keep opNames[] in sync with changes
@@ -205,6 +208,61 @@ typedef enum
 	SVC_INVERSE_MODULATE
 } stageVertexColor_t;
 
+// SP Begin
+typedef enum
+{
+	STENCIL_COMP_GREATER,
+	STENCIL_COMP_GEQUAL,
+	STENCIL_COMP_LESS,
+	STENCIL_COMP_LEQUAL,
+	STENCIL_COMP_EQUAL,
+	STENCIL_COMP_NOTEQUAL,
+	STENCIL_COMP_ALWAYS,
+	STENCIL_COMP_NEVER
+} stencilComp_t;
+
+typedef enum
+{
+	STENCIL_OP_KEEP,
+	STENCIL_OP_ZERO,
+	STENCIL_OP_REPLACE,
+	STENCIL_OP_INCRSAT,
+	STENCIL_OP_DECRSAT,
+	STENCIL_OP_INVERT,
+	STENCIL_OP_INCRWRAP,
+	STENCIL_OP_DECRWRAP
+} stencilOperation_t;
+
+typedef struct
+{
+	// The value to be compared against (if Comp is anything else than always) and/or the value to be written to the buffer
+	// (if either Pass, Fail or ZFail is set to replace).
+	byte ref = 0;
+
+	// An 8 bit mask as an 0255 integer, used when comparing the reference value with the contents of the buffer
+	// (referenceValue & readMask) comparisonFunction (stencilBufferValue & readMask).
+	byte readMask = 255;
+
+	// An 8 bit mask as an 0255 integer, used when writing to the buffer.Note that, like other write masks,
+	// it specifies which bits of stencil buffer will be affected by write
+	// (i.e.WriteMask 0 means that no bits are affected and not that 0 will be written).
+	byte writeMask = 255;
+
+	// Function used to compare the reference value to the current contents of the buffer.
+	stencilComp_t comp = STENCIL_COMP_ALWAYS;
+
+	// What to do with the contents of the buffer if the stencil test(and the depth test) passes.
+	stencilOperation_t pass = STENCIL_OP_KEEP;
+
+	// What to do with the contents of the buffer if the stencil test fails.
+	stencilOperation_t fail = STENCIL_OP_KEEP;
+
+	// What to do with the contents of the buffer if the stencil test passes, but the depth test fails.
+	stencilOperation_t zFail = STENCIL_OP_KEEP;
+} stencilStage_t;
+// SP End
+
+
 static const int	MAX_FRAGMENT_IMAGES = 8;
 static const int	MAX_VERTEX_PARMS = 4;
 
@@ -234,6 +292,7 @@ typedef struct
 	// if the surface is alpha tested
 	float				privatePolygonOffset;	// a per-stage polygon offset
 
+	stencilStage_t*     stencilStage;
 	newShaderStage_t*	newStage;			// vertex / fragment program based stage
 } shaderStage_t;
 
@@ -267,6 +326,13 @@ typedef enum
 	SS_POST_PROCESS = 100	// after a screen copy to texture
 } materialSort_t;
 
+enum SubViewType : uint16_t
+{
+	SUBVIEW_NONE,
+	SUBVIEW_MIRROR,
+	SUBVIEW_DIRECT_PORTAL
+};
+
 typedef enum
 {
 	CT_FRONT_SIDED,
@@ -377,6 +443,7 @@ typedef enum
 								  // won't collect light from any angle
 } surfaceFlags_t;
 
+
 class idSoundEmitter;
 
 // RB: predefined Quake 1 light styles
@@ -497,6 +564,16 @@ public:
 		return hasSubview;
 	}
 
+	bool                IsPortalSubView() const
+	{
+		return subViewType == SubViewType::SUBVIEW_DIRECT_PORTAL;
+	}
+
+	bool                IsMirrorSubView() const
+	{
+		return subViewType == SubViewType::SUBVIEW_MIRROR;
+	}
+
 	// returns true if the material will generate shadows, not making a
 	// distinction between global and no-self shadows
 	bool				SurfaceCastsShadow() const
@@ -882,6 +959,9 @@ private:
 	void				ParseVertexParm( idLexer& src, newShaderStage_t* newStage );
 	void				ParseVertexParm2( idLexer& src, newShaderStage_t* newStage );
 	void				ParseFragmentMap( idLexer& src, newShaderStage_t* newStage );
+	void                ParseStencilCompare( const idToken& token, stencilComp_t* stencilComp );
+	void                ParseStencilOperation( const idToken& token, stencilOperation_t* stencilOp );
+	void                ParseStencil( idLexer& src, stencilStage_t* stencilStage );
 	void				ParseStage( idLexer& src, const textureRepeat_t trpDefault = TR_REPEAT );
 	void				ParseDeform( idLexer& src );
 	void				ParseDecalInfo( idLexer& src );
@@ -939,6 +1019,7 @@ private:
 
 	materialCoverage_t	coverage;
 	cullType_t			cullType;			// CT_FRONT_SIDED, CT_BACK_SIDED, or CT_TWO_SIDED
+	SubViewType         subViewType;        // SP added
 	bool				shouldCreateBackSides;
 
 	bool				fogLight;
