@@ -455,13 +455,44 @@ void Framebuffer::AddDepthBuffer( int format, int multiSamples )
 
 	if( notCreatedYet )
 	{
-		glFramebufferRenderbuffer( GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depthBuffer );
+		glFramebufferRenderbuffer( GL_FRAMEBUFFER, GL_DEPTH_STENCIL_ATTACHMENT, GL_RENDERBUFFER, depthBuffer );
 	}
 
 	GL_CheckErrors();
 }
 
-void Framebuffer::AttachImage2D( int target, const idImage* image, int index, int mipmapLod )
+void Framebuffer::AddStencilBuffer( int format, int multiSamples )
+{
+	stencilFormat = format;
+
+	bool notCreatedYet = stencilBuffer == 0;
+	if( notCreatedYet )
+	{
+		glGenRenderbuffers( 1, &stencilBuffer );
+	}
+
+	glBindRenderbuffer( GL_RENDERBUFFER, stencilBuffer );
+
+	if( multiSamples > 0 )
+	{
+		glRenderbufferStorageMultisample( GL_RENDERBUFFER, multiSamples, format, width, height );
+
+		msaaSamples = true;
+	}
+	else
+	{
+		glRenderbufferStorage( GL_RENDERBUFFER, format, width, height );
+	}
+
+	if( notCreatedYet )
+	{
+		glFramebufferRenderbuffer( GL_FRAMEBUFFER, GL_STENCIL_ATTACHMENT, GL_RENDERBUFFER, stencilBuffer );
+	}
+
+	GL_CheckErrors();
+}
+
+void Framebuffer::AttachImage2D( int target, idImage* image, int index, int mipmapLod )
 {
 	if( ( target != GL_TEXTURE_2D ) && ( target != GL_TEXTURE_2D_MULTISAMPLE ) && ( target < GL_TEXTURE_CUBE_MAP_POSITIVE_X || target > GL_TEXTURE_CUBE_MAP_NEGATIVE_Z ) )
 	{
@@ -476,9 +507,11 @@ void Framebuffer::AttachImage2D( int target, const idImage* image, int index, in
 	}
 
 	glFramebufferTexture2D( GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0 + index, target, image->texnum, mipmapLod );
+
+	image->opts.isRenderTarget = true;
 }
 
-void Framebuffer::AttachImageDepth( int target, const idImage* image )
+void Framebuffer::AttachImageDepth( int target, idImage* image )
 {
 	if( ( target != GL_TEXTURE_2D ) && ( target != GL_TEXTURE_2D_MULTISAMPLE ) )
 	{
@@ -486,12 +519,16 @@ void Framebuffer::AttachImageDepth( int target, const idImage* image )
 		return;
 	}
 
-	glFramebufferTexture2D( GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, target, image->texnum, 0 );
+	glFramebufferTexture2D( GL_FRAMEBUFFER, GL_DEPTH_STENCIL_ATTACHMENT, target, image->texnum, 0 );
+
+	image->opts.isRenderTarget = true;
 }
 
-void Framebuffer::AttachImageDepthLayer( const idImage* image, int layer )
+void Framebuffer::AttachImageDepthLayer( idImage* image, int layer )
 {
 	glFramebufferTextureLayer( GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, image->texnum, 0, layer );
+
+	image->opts.isRenderTarget = true;
 }
 
 void Framebuffer::Check()
