@@ -1074,6 +1074,7 @@ void idRenderWorldLocal::RenderScene( const renderView_t* renderView )
 	// setup view parms for the initial view
 	viewDef_t* parms = ( viewDef_t* )R_ClearedFrameAlloc( sizeof( *parms ), FRAME_ALLOC_VIEW_DEF );
 	parms->renderView = *renderView;
+	parms->targetRender = nullptr;
 
 	if( tr.takingScreenshot )
 	{
@@ -1102,6 +1103,7 @@ void idRenderWorldLocal::RenderScene( const renderView_t* renderView )
 	parms->scissor.y2 = parms->viewport.y2 - parms->viewport.y1;
 
 	parms->isSubview = false;
+	parms->isObliqueProjection = false;
 	parms->initialViewAreaOrigin = renderView->vieworg;
 	parms->renderWorld = this;
 
