@@ -29,7 +29,7 @@ If you have questions concerning this license or the applicable additional terms
 #include "precompiled.h"
 #pragma hdrstop
 #include "../renderer/Image.h"
-#include "../renderer/DXT//DXTCodec.h"
+#include "../renderer/DXT/DXTCodec.h"
 
 #pragma warning(disable: 4355) // 'this' : used in base member initializer list
 
@@ -189,7 +189,7 @@ idSWF::idSWF( const char* filename_, idSoundWorld* soundWorld_ )
 	}
 
 	idStr atlasFileName = binaryFileName;
-	atlasFileName.SetFileExtension( ".tga" );
+	atlasFileName.SetFileExtension( ".png" );
 	atlasMaterial = declManager->FindMaterial( atlasFileName );
 
 	byte* atlasExportImageRGBA = NULL;
@@ -198,112 +198,123 @@ idSWF::idSWF( const char* filename_, idSoundWorld* soundWorld_ )
 
 	if( /*!loadedFromJSON &&*/ ( postLoadExportFlashToJSON.GetBool() || postLoadExportFlashAtlas.GetBool() || postLoadExportFlashToSWF.GetBool() ) )
 	{
-		idStrStatic< MAX_OSPATH > generatedName = atlasFileName;
-		generatedName.StripFileExtension();
-		idImage::GetGeneratedName( generatedName, TD_DEFAULT, CF_2D );
+		// try loading the TGA first
+		ID_TIME_T timestamp;
+		//LoadTGA( atlasFileName.c_str(), &atlasExportImageRGBA, &atlasExportImageWidth, &atlasExportImageHeight, &timestamp );
+		LoadPNG( atlasFileName.c_str(), &atlasExportImageRGBA, &atlasExportImageWidth, &atlasExportImageHeight, &timestamp );
 
-		idBinaryImage im( generatedName );
-		ID_TIME_T binaryFileTime = im.LoadFromGeneratedFile( FILE_NOT_FOUND_TIMESTAMP );
-
-		if( binaryFileTime != FILE_NOT_FOUND_TIMESTAMP )
+		if( ( atlasExportImageRGBA == NULL ) || ( timestamp == FILE_NOT_FOUND_TIMESTAMP ) )
 		{
-			const bimageFile_t& imgHeader = im.GetFileHeader();
-			const bimageImage_t& img = im.GetImageHeader( 0 );
+			idLib::Warning( "failed to load atlas '%s'", atlasFileName.c_str() );
 
-			const byte* data = im.GetImageData( 0 );
+			idStrStatic< MAX_OSPATH > generatedName = atlasFileName;
+			generatedName.StripFileExtension();
+			idImage::GetGeneratedName( generatedName, TD_DEFAULT, CF_2D );
 
-			//( img.level, 0, 0, img.destZ, img.width, img.height, data );
+			idBinaryImage im( generatedName );
+			ID_TIME_T binaryFileTime = im.LoadFromGeneratedFile( FILE_NOT_FOUND_TIMESTAMP );
 
-			idTempArray<byte> rgba( img.width * img.height * 4 );
-			memset( rgba.Ptr(), 255, rgba.Size() );
-
-			if( imgHeader.format == FMT_DXT1 )
+			if( binaryFileTime != FILE_NOT_FOUND_TIMESTAMP )
 			{
-				idDxtDecoder dxt;
-				dxt.DecompressImageDXT1( data, rgba.Ptr(), img.width, img.height );
-			}
-			else if( imgHeader.format == FMT_DXT5 )
-			{
-				idDxtDecoder dxt;
+				const bimageFile_t& imgHeader = im.GetFileHeader();
+				const bimageImage_t& img = im.GetImageHeader( 0 );
 
-				if( imgHeader.colorFormat == CFM_NORMAL_DXT5 )
+				const byte* data = im.GetImageData( 0 );
+
+				//( img.level, 0, 0, img.destZ, img.width, img.height, data );
+
+				idTempArray<byte> rgba( img.width * img.height * 4 );
+				memset( rgba.Ptr(), 255, rgba.Size() );
+
+				if( imgHeader.format == FMT_DXT1 )
 				{
-					dxt.DecompressNormalMapDXT5( data, rgba.Ptr(), img.width, img.height );
+					idDxtDecoder dxt;
+					dxt.DecompressImageDXT1( data, rgba.Ptr(), img.width, img.height );
 				}
-				else if( imgHeader.colorFormat == CFM_YCOCG_DXT5 )
+				else if( imgHeader.format == FMT_DXT5 )
 				{
-					dxt.DecompressYCoCgDXT5( data, rgba.Ptr(), img.width, img.height );
+					idDxtDecoder dxt;
+
+					if( imgHeader.colorFormat == CFM_NORMAL_DXT5 )
+					{
+						dxt.DecompressNormalMapDXT5( data, rgba.Ptr(), img.width, img.height );
+					}
+					else if( imgHeader.colorFormat == CFM_YCOCG_DXT5 )
+					{
+						dxt.DecompressYCoCgDXT5( data, rgba.Ptr(), img.width, img.height );
+					}
+					else
+					{
+
+						dxt.DecompressImageDXT5( data, rgba.Ptr(), img.width, img.height );
+					}
+				}
+				else if( imgHeader.format == FMT_LUM8 || imgHeader.format == FMT_INT8 )
+				{
+					// LUM8 and INT8 just read the red channel
+					byte* pic = rgba.Ptr();
+					for( int i = 0; i < img.dataSize; i++ )
+					{
+						pic[ i * 4 ] = data[ i ];
+					}
+				}
+				else if( imgHeader.format == FMT_ALPHA )
+				{
+					// ALPHA reads the alpha channel
+					byte* pic = rgba.Ptr();
+					for( int i = 0; i < img.dataSize; i++ )
+					{
+						pic[ i * 4 + 3 ] = data[ i ];
+					}
+				}
+				else if( imgHeader.format == FMT_L8A8 )
+				{
+					// L8A8 reads the alpha and red channels
+					byte* pic = rgba.Ptr();
+					for( int i = 0; i < img.dataSize / 2; i++ )
+					{
+						pic[ i * 4 + 0 ] = data[ i * 2 + 0 ];
+						pic[ i * 4 + 3 ] = data[ i * 2 + 1 ];
+					}
+				}
+				else if( imgHeader.format == FMT_RGB565 )
+				{
+					// FIXME
+					/*
+					byte* pic = rgba.Ptr();
+					for( int i = 0; i < img.dataSize / 2; i++ )
+					{
+						unsigned short color = ( ( pic[ i * 4 + 0 ] >> 3 ) << 11 ) | ( ( pic[ i * 4 + 1 ] >> 2 ) << 5 ) | ( pic[ i * 4 + 2 ] >> 3 );
+						img.data[ i * 2 + 0 ] = ( color >> 8 ) & 0xFF;
+						img.data[ i * 2 + 1 ] = color & 0xFF;
+					}
+					*/
 				}
 				else
 				{
+					byte* pic = rgba.Ptr();
+					for( int i = 0; i < img.dataSize; i++ )
+					{
+						pic[ i ] = data[ i ];
+					}
+				}
 
-					dxt.DecompressImageDXT5( data, rgba.Ptr(), img.width, img.height );
-				}
-			}
-			else if( imgHeader.format == FMT_LUM8 || imgHeader.format == FMT_INT8 )
-			{
-				// LUM8 and INT8 just read the red channel
-				byte* pic = rgba.Ptr();
-				for( int i = 0; i < img.dataSize; i++ )
-				{
-					pic[ i * 4 ] = data[ i ];
-				}
-			}
-			else if( imgHeader.format == FMT_ALPHA )
-			{
-				// ALPHA reads the alpha channel
-				byte* pic = rgba.Ptr();
-				for( int i = 0; i < img.dataSize; i++ )
-				{
-					pic[ i * 4 + 3 ] = data[ i ];
-				}
-			}
-			else if( imgHeader.format == FMT_L8A8 )
-			{
-				// L8A8 reads the alpha and red channels
-				byte* pic = rgba.Ptr();
-				for( int i = 0; i < img.dataSize / 2; i++ )
-				{
-					pic[ i * 4 + 0 ] = data[ i * 2 + 0 ];
-					pic[ i * 4 + 3 ] = data[ i * 2 + 1 ];
-				}
-			}
-			else if( imgHeader.format == FMT_RGB565 )
-			{
-				// FIXME
-				/*
-				byte* pic = rgba.Ptr();
-				for( int i = 0; i < img.dataSize / 2; i++ )
-				{
-					unsigned short color = ( ( pic[ i * 4 + 0 ] >> 3 ) << 11 ) | ( ( pic[ i * 4 + 1 ] >> 2 ) << 5 ) | ( pic[ i * 4 + 2 ] >> 3 );
-					img.data[ i * 2 + 0 ] = ( color >> 8 ) & 0xFF;
-					img.data[ i * 2 + 1 ] = color & 0xFF;
-				}
-				*/
-			}
-			else
-			{
-				byte* pic = rgba.Ptr();
-				for( int i = 0; i < img.dataSize; i++ )
-				{
-					pic[ i ] = data[ i ];
-				}
-			}
+				idStr atlasFileNameExport = atlasFileName;
+				atlasFileNameExport.Replace( "generated/", "exported/" );
+				atlasFileNameExport.SetFileExtension( ".png" );
 
-			idStr atlasFileNameExport = atlasFileName;
-			atlasFileNameExport.Replace( "generated/", "exported/" );
-			atlasFileNameExport.SetFileExtension( ".png" );
+				R_WritePNG( atlasFileNameExport, rgba.Ptr(), 4, img.width, img.height, true, "fs_basepath" );
 
-			R_WritePNG( atlasFileNameExport, rgba.Ptr(), 4, img.width, img.height, true, "fs_basepath" );
-
-			if( postLoadExportFlashToSWF.GetBool() )
-			{
-				atlasExportImageWidth = img.width;
-				atlasExportImageHeight = img.height;
-				atlasExportImageRGBA = ( byte* ) Mem_Alloc( rgba.Size(), TAG_TEMP );
-				memcpy( atlasExportImageRGBA, rgba.Ptr(), rgba.Size() );
+				if( postLoadExportFlashToSWF.GetBool() )
+				{
+					atlasExportImageWidth = img.width;
+					atlasExportImageHeight = img.height;
+					atlasExportImageRGBA = ( byte* ) Mem_Alloc( rgba.Size(), TAG_TEMP );
+					memcpy( atlasExportImageRGBA, rgba.Ptr(), rgba.Size() );
+				}
 			}
 		}
+
 	}
 
 	if( postLoadExportFlashToSWF.GetBool() )
