@@ -75,15 +75,17 @@ static SDL_Window* window = nullptr;
 
 // Eric: Integrate this into RBDoom3BFG's source code ecosystem.
 // Helper function for using SDL2 and Vulkan on Linux.
-std::vector<const char*> get_required_extensions( const std::vector<const char*>& instanceExtensions, bool enableValidationLayers )
+std::vector<const char*> get_required_extensions()
 {
 	uint32_t                 sdlCount = 0;
-	std::vector<const char*> sdlInstanceExtensions;
+	std::vector<const char*> sdlInstanceExtensions = {};
 
 	SDL_Vulkan_GetInstanceExtensions( nullptr, &sdlCount, nullptr );
 	sdlInstanceExtensions.resize( sdlCount );
 	SDL_Vulkan_GetInstanceExtensions( nullptr, &sdlCount, sdlInstanceExtensions.data() );
 
+	// SRS - Report enabled instance extensions in CreateVulkanInstance() vs. doing it here
+	/*
 	if( enableValidationLayers )
 	{
 		idLib::Printf( "\nNumber of availiable instance extensions\t%i\n", sdlCount );
@@ -93,6 +95,7 @@ std::vector<const char*> get_required_extensions( const std::vector<const char*>
 			idLib::Printf( "\t%s\n", ext );
 		}
 	}
+	*/
 
 	// SRS - needed for MoltenVK portability implementation and optionally for MoltenVK configuration on OSX
 #if defined(__APPLE__)
@@ -102,6 +105,8 @@ std::vector<const char*> get_required_extensions( const std::vector<const char*>
 #endif
 #endif
 
+	// SRS - Add debug instance extensions in CreateVulkanInstance() vs. hardcoding them here
+	/*
 	if( enableValidationLayers )
 	{
 		sdlInstanceExtensions.push_back( "VK_EXT_debug_report" );
@@ -114,6 +119,7 @@ std::vector<const char*> get_required_extensions( const std::vector<const char*>
 			idLib::Printf( "\t%s\n", ext );
 		}
 	}
+	*/
 
 	return sdlInstanceExtensions;
 }
